import { container_paths, containers_path } from "./paths.ts";
import { exists, log_from } from "./utils.ts";

const log = log_from("config");

export type ContainerConfig = ReturnType<typeof new_container_config>;
export function new_container_config(name: string) {
	return {
		name,
		version: 0,
		redirects: [] as [number, number][],
	};
}

export async function load_container_config(name: string) {
	const config_path = container_paths(name).configuration;
	log("loading config for", name);
	if (!exists(config_path)) return null;
	const content = await Deno.readTextFile(config_path);
	const read = JSON.parse(content);
	const default_ = new_container_config(name);
	if (read.version < default_.version) throw new Error("read conf version is outdated");
	return { ...default_, ...read } as ContainerConfig;
}

export async function save_container_config(config: ContainerConfig) {
	log("saving config of", config.name);
	const config_path = container_paths(config.name).configuration;
	const serialized = JSON.stringify(config, null, 4);
	await Deno.writeTextFile(config_path, serialized);
}

export async function load_all_configs() {
	const names = Array.from(Deno.readDirSync(containers_path()))
		.filter((e) => e.isDirectory)
		.map((e) => e.name);
	const results = await Promise.all(names.map((name) => load_container_config(name)));
	return results.filter((item) => item != null) as ContainerConfig[];
}
